package geniusweb.bidspace.pareto;

import java.io.IOException;
import java.math.BigDecimal;
import java.util.Collection;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.apache.commons.math.stat.descriptive.DescriptiveStatistics;
import org.junit.Test;

import geniusweb.issuevalue.DiscreteValue;
import geniusweb.issuevalue.DiscreteValueSet;
import geniusweb.issuevalue.Domain;
import geniusweb.issuevalue.ValueSet;
import geniusweb.profile.utilityspace.DiscreteValueSetUtilities;
import geniusweb.profile.utilityspace.LinearAdditive;
import geniusweb.profile.utilityspace.LinearAdditiveUtilitySpace;
import geniusweb.profile.utilityspace.ValueSetUtilities;

/**
 * Test pareto on random space.
 */
public class ParetoRandomTest {

	private final int DIMENSION = 2; // #participants. 2 is normal bilateral
										// space
	private final int NISSUES = 3; // number of issues
	private final int NVALS = 10; // number values per issue
	private final int NRUNS = 50; // number of runs (to average result)

	@Test
	public void checkGenerciPareto() throws IOException {

		DescriptiveStatistics time = new DescriptiveStatistics();
		DescriptiveStatistics paretopoints = new DescriptiveStatistics();

		for (int run = 0; run < NRUNS; run++) {
			System.gc(); // free memory before each test run
			System.out.print(".");
			long startTime = System.nanoTime();
			ParetoLinearAdditive pareto = runOneTest();
			paretopoints.addValue(pareto.getPoints().size());

			double dt = (System.nanoTime() - startTime) / 1000000000d;

			time.addValue(dt);
		}
		System.out.println();
		System.out.println("Results averaged over " + NRUNS + " runs");
		System.out.println("DIMENSION=" + DIMENSION);
		System.out.println("NISSUES=" + NISSUES);
		System.out.println("NVALS=" + NVALS);
		System.out.println("bidspace=" + Math.pow(NVALS, NISSUES));
		System.out.println("runtime: " + time.getMean() + "s, stdev="
				+ time.getStandardDeviation());
		System.out.println("points on pareto: " + paretopoints.getMean()
				+ ", stdev=" + paretopoints.getStandardDeviation());
		System.out.println("{" + DIMENSION + "," + NISSUES + "," + NVALS + ","
				+ time.getMean() + "," + time.getStandardDeviation() + ","
				+ paretopoints.getMean() + ","
				+ paretopoints.getStandardDeviation() + "}");
	}

	private ParetoLinearAdditive runOneTest() {
		Domain domain = getRandomDomain();

		List<LinearAdditive> profiles = new LinkedList<>();
		for (int n = 0; n < DIMENSION; n++) {
			LinearAdditiveUtilitySpace rspace = getRandomSpace(domain);
			profiles.add(rspace);
		}
		return new ParetoLinearAdditive(profiles);

	}

	private Domain getRandomDomain() {
		Map<String, ValueSet> issues = new HashMap<>();
		List<BigDecimal> w = getRandomWeights();
		for (int iss = 0; iss < NISSUES; iss++) {
			Collection<DiscreteValue> values = new LinkedList<>();
			for (int val = 0; val < NVALS; val++) {
				DiscreteValue v = new DiscreteValue("iss" + iss + "val" + val);
				values.add(v);
			}
			String issuename = "issue" + iss;
			issues.put(issuename, new DiscreteValueSet(values));

		}
		return new Domain("dom", issues);
	}

	/**
	 * 
	 * @param domain some domain with discrete issue values
	 * @return random LA utilspace for this domain
	 */
	private LinearAdditiveUtilitySpace getRandomSpace(Domain domain) {
		List<String> issuelist = new LinkedList(domain.getIssues());
		Map<String, ValueSetUtilities> utils = new HashMap<>();
		Map<String, BigDecimal> weights = new HashMap<>();
		List<BigDecimal> w = getRandomWeights();
		for (int iss = 0; iss < NISSUES; iss++) {
			String issuename = issuelist.get(iss);
			Map<DiscreteValue, BigDecimal> utilsmap = new HashMap<>();
			for (int val = 0; val < NVALS; val++) {
				DiscreteValue v = (DiscreteValue) domain.getValues(issuename)
						.get(val);
				utilsmap.put(v, BigDecimal.valueOf(Math.random()));
			}
			weights.put(issuename, w.get(iss));
			utils.put(issuename, new DiscreteValueSetUtilities(utilsmap));
		}
		return new LinearAdditiveUtilitySpace(domain, "name", utils, weights,
				null);
	}

	/**
	 * 
	 * @return list of NISSUES BigDecimals, summing to 1
	 */
	private List<BigDecimal> getRandomWeights() {
		List<BigDecimal> doubles = new LinkedList<>();
		BigDecimal remainder = BigDecimal.ONE;
		for (int n = 0; n < NISSUES - 1; n++) {
			BigDecimal next = BigDecimal.valueOf(Math.random() / NISSUES);
			doubles.add(next);
			remainder = remainder.subtract(next);
		}
		doubles.add(remainder);
		return doubles;
	}

}
